@@ -1,10 +1,11 @@
-from argparse import ArgumentParser
+from argparse import ArgumentParser, _SubParsersAction
 from collections import OrderedDict
 from copy import deepcopy
 import json
 from pprint import pformat
 import sys
 import time
+from warnings import warn
 from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, TypeVar, Union, get_type_hints
 from typing_inspect import is_literal_type, get_args, get_origin, is_union_type
@@ -62,6 +63,9 @@ class Tap(ArgumentParser):
                               with any capitalization as well as 1 or 0.
         :param kwargs: Keyword arguments passed to the super class ArgumentParser.
         """
+        # Whether the Tap object has been initialized
+        self._initialized = False
+
         # Whether boolean flags have to be explicitly set to True or False
         self._explicit_bool = explicit_bool
 
@@ -77,6 +81,9 @@ class Tap(ArgumentParser):
         # Create argument buffer
         self.argument_buffer = OrderedDict()
 
+        # Create a place to put all of the subparsers
+        self._subparser_buffer: List[Tuple[str, type, Dict[str, Any]]] = []
+
         # Get class variables help strings from the comments
         self.class_variables = self._get_class_variables()
 
@@ -86,9 +93,14 @@ class Tap(ArgumentParser):
         # Initialize the super class, i.e. ArgumentParser
         super(Tap, self).__init__(*args, **kwargs)
 
-        # Add arguments to self
-        self.add_arguments()  # Adds user-overridden arguments to the arguments buffer
-        self._add_arguments()  # Adds all arguments in order to self
+        # Stores all of the subparsers
+        self._subparsers = None
+
+        # Perform additional configuration such as modifying add_arguments or adding subparsers
+        self._configure()
+
+        # Indicate that initialization is complete
+        self._initialized = True
 
     def _add_argument(self, *name_or_flags, **kwargs) -> None:
         """Adds an argument to self (i.e. the super class ArgumentParser).
@@ -230,6 +242,11 @@ class Tap(ArgumentParser):
 
     def add_argument(self, *name_or_flags, **kwargs) -> None:
         """Adds an argument to the argument buffer, which will later be passed to _add_argument."""
+        if self._initialized:
+            raise ValueError('add_argument cannot be called after initialization. '
+                             'Arguments must be added either as class variables or by overriding '
+                             'add_arguments and including a self.add_argument call there.')
+
         variable = get_argument_name(*name_or_flags)
         self.argument_buffer[variable] = (name_or_flags, kwargs)
 
@@ -249,13 +266,65 @@ class Tap(ArgumentParser):
                 self._add_argument(*name_or_flags, **kwargs)
 
     def add_arguments(self) -> None:
-        """Explicitly add arguments to the argument buffer if not using default settings."""
+        """Deprecated and will be removed by January 1st, 2021.
+
+        Explicitly add arguments to the argument buffer if not using default settings.
+        """
         pass
 
     def process_args(self) -> None:
         """Perform additional argument processing and/or validation."""
         pass
 
+    def add_subparser(self, flag: str, subparser_type: type, **kwargs) -> None:
+        """Add a subparser to the collection of subparsers"""
+        self._subparser_buffer.append((flag, subparser_type, kwargs))
+
+    def _add_subparsers(self) -> None:
+        """Add each of the subparsers to the Tap object. """
+        # Initialize the _subparsers object if not already created
+        if self._subparsers is None and len(self._subparser_buffer) > 0:
+            self._subparsers = super(Tap, self).add_subparsers()
+
+        # Load each subparser
+        for flag, subparser_type, kwargs in self._subparser_buffer:
+            self._subparsers._parser_class = subparser_type
+            self._subparsers.add_parser(flag, **kwargs)
+
+    def add_subparsers(self, **kwargs) -> None:
+        self._subparsers = super().add_subparsers(**kwargs)
+
+    def _configure(self) -> None:
+        """Executes the user-defined configuration. """
+        # Call the user-defined configuration
+        self.configure()
+
+        # Support the previous add_arguments interface with a deprecation warning
+        if self.__class__.add_arguments != Tap.add_arguments:
+            warn('add_arguments is deprecated and will be removed on January 1st, 2021'
+                 ' please override "configure" instead and call add_argument there. ')
+            self.add_arguments()
+
+        # Add arguments to self
+        self._add_arguments()
+
+        # Add subparsers to self
+        self._add_subparsers()
+
+    def configure(self) -> None:
+        """Overwrite this method to configure the parser during initialization.
+
+        For example,
+            self.add_argument('--sum',
+                              dest='accumulate',
+                              action='store_const',
+                              const=sum,
+                              default=max)
+            self.add_subparsers(help='sub-command help')
+            self.add_subparser('a', SubparserA, help='a help')
+        """
+        pass
+
     @staticmethod
     def get_reproducibility_info() -> Dict[str, str]:
         """Gets a dictionary of reproducibility information.
@@ -304,6 +373,10 @@ class Tap(ArgumentParser):
         Unparsed arguments are saved to self.extra_args.
         :return: self, which is a Tap instance containing all of the parsed args.
         """
+        # Prevent double parsing
+        if self._parsed:
+            raise ValueError('parse_args can only be called once.')
+
         # Parse args using super class ArgumentParser's parse_args or parse_known_args function
         if known_only:
             default_namespace, self.extra_args = super(Tap, self).parse_known_args(args)
